import { x25519 } from "@noble/curves/ed25519";
import { defer } from "@lumeweb/libkernel/module";
import { KernelAuthStatus } from "@lumeweb/libkernel";

let queriesNonce = 1;
let queries: any = {};
let authStatus: KernelAuthStatus;
let authStatusKnown = false;
let authStatusDefer = defer();
let loggedInDefer = defer();

const store = new Map<string, any>(
  Object.entries({
    loginComplete: false,
    logoutComplete: false,
    kernelLoaded: "not yet",
    communicationKey: null,
    frontendCommunicationPubKey: null,
  }),
);

export function setLoginComplete(status: boolean) {
  store.set("loginComplete", status);
}
export function getLoginComplete(): boolean {
  return store.get("loginComplete");
}
export function setLogoutComplete(status: boolean) {
  store.set("logoutComplete", status);
}
export function getLogoutComplete(): boolean {
  return store.get("logoutComplete");
}
export function setKernelLoaded(status: string) {
  store.set("kernelLoaded", status);
}

export function getKernelLoaded(): string {
  return store.get("kernelLoaded");
}

export function getCommunicationKey(): Uint8Array {
  if (!store.get("communicationKey")) {
    store.set("communicationKey", x25519.utils.randomPrivateKey());
  }

  return store.get("communicationKey");
}

export function getCommunicationPubKey() {
  return x25519.getPublicKey(getCommunicationKey());
}

export function getFrontendCommunicationPubkey(): Uint8Array {
  return store.get("frontendCommunicationPubKey");
}

export function setFrontendCommunicationPubkey(key: Uint8Array) {
  store.set("frontendCommunicationPubKey", key);
}
export function getAuthStatusDefer() {
  return authStatusDefer;
}
export function getQueriesNonce(): number {
  return queriesNonce;
}
export function addQuery(nonce: any, func: Function) {
  queries[nonce] = func;
}

export function increaseQueriesNonce() {
  queriesNonce++;
}
export function setAuthStatus(status: KernelAuthStatus) {
  authStatus = status;
}

export function getAuthStatusKnown() {
  return authStatusKnown;
}
export function setAuthStatusKnown(status: boolean) {
  authStatusKnown = status;
}
export function getAuthStatus(): KernelAuthStatus {
  return authStatus;
}
export function getLoggedInDefer() {
  return loggedInDefer;
}
export function resetLoggedInDefer() {
  loggedInDefer = defer();
}
export function getQueries() {
  return queries;
}

export function getQuery(nonce: any) {
  return queries[nonce];
}

export function deleteQuery(nonce: any) {
  delete queries[nonce];
}
